@@ -44,15 +44,29 @@ def _get_public_suffix_list():
 PSL = _get_public_suffix_list()
 
 
+SCHEMES = ['http', 'https', 'ftp']
+
 def normalize(url):
     parts = extract(url)
-    nurl = parts.scheme + '://' if parts.scheme else ''
-    nurl += parts.subdomain + '.' if parts.subdomain else ''
+    nurl = ''
+    if parts.scheme:
+        if parts.scheme in SCHEMES:
+            nurl += parts.scheme + '://'
+        else:
+            nurl += parts.scheme + ':'
+    if parts.username and parts.password:
+        nurl += parts.username + ':' + parts.password + '@'
+    if parts.subdomain:
+        nurl += parts.subdomain + '.'
     nurl += parts.domain
-    nurl += "." + parts.tld
+    if parts.tld:
+        nurl += '.' + parts.tld
     if parts.port and parts.port != '80':
         nurl += ':' + parts.port
-    nurl += normpath(parts.path) if parts.path else ''
+    if parts.path:
+        nurl += normpath(parts.path)
+    elif parts.scheme in SCHEMES:
+        nurl += '/'
     if parts.query:
         nurl += '?' + parts.query
     if parts.fragment:
@@ -62,14 +76,18 @@ def normalize(url):
 
 PORT_RE = re.compile(r'(?<=.:)[1-9]+[0-9]{0,4}$')
 SCHEME_RE = re.compile(r'^[a-zA-Z]+:(//)?')
-SplitResult = namedtuple('SplitResult', 'scheme netloc path query fragment')
-ParseResult = namedtuple('ParseResult', 'scheme subdomain domain tld port path query fragment')
+USER_RE = re.compile(r'^[^:@]*:[^:@]*@.*$')
+SplitResult = namedtuple('SplitResult', ['scheme', 'netloc', 'path', 'query',
+                                         'fragment'])
+ParseResult = namedtuple('ParseResult', ['scheme', 'username', 'password',
+                                         'subdomain', 'domain', 'tld', 'port',
+                                         'path', 'query', 'fragment'])
 
 def split(url):
     """Split URLs into scheme, netloc, path, query and fragment
     """
     scheme = netloc = path = query = fragment = ''
-    if SCHEME_RE.findall(url):
+    if SCHEME_RE.search(url):
         l = url.find(':')
         scheme = url[:l].lower()
         rest = url[l:].lstrip(':/')
@@ -104,54 +122,60 @@ def _clean_netloc(netloc):
     return netloc.rstrip('.:').decode('utf-8').lower().encode('utf-8')
 
 def _split_netloc(netloc):
+    username = password = subdomain = tld = port = ''
+    if USER_RE.search(netloc):
+        user_pw, netloc = netloc.split('@', 1)
+        username, password = user_pw.split(':', 1)
     netloc = _clean_netloc(netloc)
-    subdomain = ''
-    domain = netloc
-    tld = ''
-    port = ''
-    if PORT_RE.findall(netloc):
+    if '.' not in netloc:
+        return '', '', '', netloc, '', ''
+    if PORT_RE.search(netloc):
         domain, port = netloc.split(':')
-    d = domain.split('.')
-    for i in range(len(d)):
-        tld = '.'.join(d[i:])
+    else:
+        domain = netloc
+    parts = domain.split('.')
+    for i in range(len(parts)):
+        tld = '.'.join(parts[i:])
         wildcard_tld = '*.' + tld
         exception_tld = '!' + tld
         if tld in PSL:
-            domain = '.'.join(d[:i])
+            domain = '.'.join(parts[:i])
             break
         if wildcard_tld in PSL:
-            domain = '.'.join(d[:i-1])
-            tld = '.'.join(d[i-1:])
+            domain = '.'.join(parts[:i-1])
+            tld = '.'.join(parts[i-1:])
             break
         if exception_tld in PSL:
-            domain = '.'.join(d[:i-1])
-            tld = '.'.join(d[i-1:])
+            domain = '.'.join(parts[:i-1])
+            tld = '.'.join(parts[i-1:])
             break
-    if domain.find('.') > 0:
+    if '.' in domain:
         (subdomain, domain) = domain.rsplit('.', 1) 
-    return subdomain, domain, tld, port
+    return username, password, subdomain, domain, tld, port
 
 def parse(url):
     parts = split(url)
     if parts.scheme:
         netloc = parts.netloc
-        (subdomain, domain, tld, port) = _split_netloc(netloc)
+        (username, password, subdomain, domain, tld, port) = _split_netloc(netloc)
     else:
-        subdomain = domain = tld = port = ''
-    path = parts.path if parts.path else '/'
-    return ParseResult(parts.scheme, subdomain, domain, tld, port, path, parts.query, parts.fragment)
+        username = password = subdomain = domain = tld = port = ''
+    path = parts.path if parts.path else ''
+    return ParseResult(parts.scheme, username, password, subdomain, domain, tld,
+                       port, path, parts.query, parts.fragment)
 
 def extract(url):
     parts = split(url)
     if parts.scheme:
         netloc = parts.netloc
-        path = parts.path if parts.path else '/'
+        path = parts.path if parts.path else ''
     else:
         netloc = parts.path
         path = ''
-        if netloc.find('/') > 0:
-            res = netloc.split('/', 1)
-            netloc = res[0]
-            path = '/' + res[1]
-    (subdomain, domain, tld, port) = _split_netloc(netloc)
-    return ParseResult(parts.scheme, subdomain, domain, tld, port, path, parts.query, parts.fragment)
+        if '/' in netloc:
+            tmp = netloc.split('/', 1)
+            netloc = tmp[0]
+            path = '/' + tmp[1]
+    (username, password, subdomain, domain, tld, port) = _split_netloc(netloc)
+    return ParseResult(parts.scheme, username, password, subdomain, domain, tld,
+                       port, path, parts.query, parts.fragment)
